#! /bin/bash

#AAC-ELD decoder Refsoft user instructions:

#This document describes how to checkout, compile and apply the MPEG reference software in order to decode AAC-ELD stereo 
#bitstreams which contain Low Delay MPEG surround (LD-MPS) extension payload. It is essential to use bitstreams which 
#contain LD-MPS extension payload length information, otherwise the reference decoder will not be able to decode as it is 
#lacking an adequate payload parser. Two software packages AAC-ELD and SAOC are needed. The MPEG Reference software uses 
#QMF data interface between both codecs in case SBR is used. If SBR is not used, the interface will be done in the time domain.

UNAME=`uname`
if [ ${UNAME} != "Linux" ]; then
    echo "This is not a linux system. Please switch machine"
    exit;
fi
CPWD=`pwd`


echo "##################################################################"
echo " PLEASE NOTE"
echo "   - that you need to have the AFsp library available," 
echo "     see also README_AFsp" 
echo "   - that you need to have the MPEG-4 File Format Library available"
echo "     see also README_refsoft" 
echo "##################################################################"
echo " "


MP4AUDEC=${CPWD}/../bin/mp4AudVm_Tbt_VmMp4AuDecRewrite/Linux_v2_c1_eptool_sbr_ps_mpeg12_gcc_O0d/mp4audec
SSNRCD=${CPWD}/../bin/conf_ssnrcd/Linux_gcc_O0d/ssnrcd 

SAOC_SRC_PACK=SAOC_IS2_ReferenceSoftware__20110812.zip
SAOC_SRC_PATH=${SAOC_SRC_PACK%.zip}

FOLDER_MP4DEC=${CPWD}/../mp4AudVm_Rewrite
FOLDER_SSNRCD=${CPWD}/../conf_ssnrcd

#--------------
#Checkout
#--------------

# get SAOC sources
if [ ! -e ${SAOC_SRC_PACK} ]; then
    echo "Downloading SAOC RefSoft package:"
    echo "  http://wg11.sc29.org/svn/repos/MPEG-D/trunk/SAOC/${SAOC_SRC_PACK}"
    echo "NOTE: Please use always the latest snapshot of the MPEG-D reference software"
    echo " "
    echo "Please enter your MPEG user name:"
    read USER_MPEG
    echo "Please enter your MPEG password:"
    read PASSWORD_MPEG
    wget --http-user=${USER_MPEG} --http-passwd=${PASSWORD_MPEG} http://wg11.sc29.org/svn/repos/MPEG-D/trunk/SAOC/${SAOC_SRC_PACK}
fi


#--------------
#Build
#--------------

mkdir -p RefSoft/lib/


# check mp4audec
if [ ! -e $MP4AUDEC ]; then
    make -C $FOLDER_MP4DEC mp4audec
fi

if [ -e ${CPWD}/../bin/Linux_v2_c1_eptool_sbr_ps_mpeg12_gcc_O0d/mp4audec ]; then
    MP4AUDEC=${CPWD}/../bin/Linux_v2_c1_eptool_sbr_ps_mpeg12_gcc_O0d/mp4audec
fi

if [ ! -e $MP4AUDEC ]; then
    echo "Error compiling mp4audec "
    echo "ERROR: $MP4AUDEC"
    exit;
fi

# LD-MPS decoder
if [ ! -e ${CPWD}/${SAOC_SRC_PATH}/bin/Linux_AFsp_gcc_O0d/sac_dec_standalone ]; then
    unzip -o $SAOC_SRC_PACK #SAOC_IS2_ReferenceSoftware__20110812.zip
    make -C ${SAOC_SRC_PATH}/mp4spatialdec/
fi


if [ ! -e ${CPWD}/${SAOC_SRC_PATH}/bin/Linux_AFsp_gcc_O0d/sac_dec_standalone ]; then
    echo "Error compiling SAOC decoder"
    echo "ERROR: ${CPWD}/${SAOC_SRC_PATH}/bin/Linux_AFsp_gcc_O0d/sac_dec_standalone"
    exit;
fi

# building ssnrcd
if [ ! -e $SSNRCD ]; then
    make -C $FOLDER_SSNRCD
fi

if [ -e ${CPWD}/../bin/Linux_gcc_O0d/mpegaudconf ]; then
    SSNRCD=${CPWD}/../bin/Linux_gcc_O0d/mpegaudconf
fi

if [ ! -e $SSNRCD ]; then
    echo "Error compiling ssnrcd "
    exit;
fi

# ELDv2 deocder script
if [ -e ${CPWD}/ELDv2_decoder.sh ]; then
    rm ${CPWD}/ELDv2_decoder.sh
fi


# create ELDv2 decoder
cat > ${CPWD}/ELDv2_decoder.sh <<DELIM

#! /bin/bash

ELDDEC=$MP4AUDEC
SSNRCD=$SSNRCD
SAOCDEC=${CPWD}/${SAOC_SRC_PATH}/bin/Linux_AFsp_gcc_O0d/sac_dec_standalone

if [ \$# -lt 2 ] || [ \$# -gt 3 ]; then
    echo "Usage: $0 input.mp4 output.wav"
    exit
fi
ulimit -Ss 32767
if [ -e aot.txt ]; then rm aot.txt; fi

\$ELDDEC \$1 -pay tmp.payload -qmf tmp.qmf -o tmp.wav -d 3 >& tmp.txt 
cat tmp.txt | grep -m 1 audioObjType | sed 's/audioSpC->audioObjType//' | sed 's/[ :]//g' > aot.txt
cat tmp.txt | grep -m 1 ldSbrPresent | sed 's/audioSpC->eldSC.ldSbrPresent//' | sed 's/[ :]//g' > sbr.txt
echo \$CMD
AOT=\`cat aot.txt\`
SBR=\`cat sbr.txt\`
echo "AOT string is \${AOT}"
if [ \$AOT == "ErrorResilientAACLowDelay" ]; then
    echo "Decoding AAC-LD + LDMPS"
     \$SAOCDEC 0 tmp.wav tmp.payload \$2
elif [ \$SBR == "1" ]; then
    echo "Decoding AAC-ELD + SBR + LDMPS"
    \$SAOCDEC -qmfin 0 tmp.qmf tmp.payload \$2
else
    echo "Decoding AAC-ELD + LDMPS"
    \$SAOCDEC 0 tmp.wav tmp.payload \$2
fi


rm tmp.*
rm aot.txt
rm sbr.txt

DELIM
chmod 755 ${CPWD}/ELDv2_decoder.sh
cd ..

echo ""
echo "######################################################################"
echo "# Scripts finished: please check the decoder ${CPWD}/ELDv2_decoder.sh #"
echo "######################################################################"
